/* This software program is licensed subject to the GNU General Public License
(GPL). Version 2, June 1991, available at <http://www.fsf.org/copyleft/gpl.html>
*/

#include <linux/types.h>

#ifndef __bitwise /* if __leXX is not defined */
typedef __u16 __le16;
typedef __u32 __le32;
typedef __u64 __le64;
#endif

/* the workqueue interface changed for 2.6.20 */
#include <linux/workqueue.h>

#ifndef INIT_DELAYED_WORK

#undef PREPARE_WORK
#define PREPARE_WORK(_work, _func)				\
	do {							\
		/* _t is for type checking */			\
		void (*_t)(struct work_struct *work) = _func;	\
		(_work)->func = (void (*)(void *))_t;		\
		(_work)->data = _work;				\
	} while (0)

#undef INIT_WORK
#define INIT_WORK(_work, _func)					\
	do {							\
		INIT_LIST_HEAD(&(_work)->entry);		\
		(_work)->pending = 0;				\
		PREPARE_WORK((_work), (_func));			\
		init_timer(&(_work)->timer);			\
	} while (0)

struct delayed_work {
	struct work_struct work;
};

#define INIT_DELAYED_WORK(_work, _func)	    INIT_WORK(&(_work)->work, _func)
#define PREPARE_DELAYED_WORK(_work, _func)  PREPARE_WORK(&(_work)->work, _func)

#define cancel_delayed_work(_s)		cancel_delayed_work(&(_s)->work)

#define queue_delayed_work(_q,_s,_time)		\
		queue_delayed_work((_q), &(_s)->work, (_time))
#endif


#include <asm/msr.h>

#if ! defined(rdtsc)
#  if defined(__i386__) || defined(__x86_64__)
#    define rdtsc(low,high) \
        __asm__ __volatile__("rdtsc" : "=a" (low), "=d" (high))
#  else
#    error We have a problem :)
#  endif /* arch */
#endif /* rdtsc */


#include <linux/version.h>
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18))

#define IRQF_SAMPLE_RANDOM	(SA_SAMPLE_RANDOM)
#define IRQF_SHARED		(SA_SHIRQ)
#define IRQF_PROBE_SHARED	(SA_PROBEIRQ)

#endif
